********************************************************** Line 1
C
C                   November 22 2010
C      ==========================================  
C
C                  Main Program DESCON
C              ==========================
C  
C   DESCON is a subroutine dedicated to compute the minimizer of 
C   a differentiable function with a large number of variables. 
C   It is supposed that we have the algebraic expression of the
C   function and its gradient.
C
C   The algorithm is described in:
C   N. Andrei, An accelerated conjugate gradient algorithm with 
C   guaranteed descent and conjugacy conditions for large-scale 
C   unconstrained optimization.
C   ICI Technical Report, November 29, 2010.
C-------------------------------------------------------------------------
C
C   The algorithm is defined as:
C
C   x(k+1) = x(k) + alpha*d(k)
C
C   where alpha is computed by the Wolfe line search conditions.
C  
C   The search direction of this algorithm is computed as:
C                                              
C   d(k+1) = -theta(k)*g(k+1) + beta(k)*s(k) 
C
C   where beta(k) is:
C                                          
C              y(k)Tg(k+1)-t(k)(s(k)Tg(k+1))
C   beta(k) = ------------------------------- .
C                        y(k)Ts(k)
C
C   The parameters theta(k) and t(k) are determined from the following two
C   conditions:                            
C
C   1) g(k+1)T*d(k+1) = -w*||g(k+1)||^2      (descent condition)
C                                           
C   2) y(k)T*d(k+1)   = -v*(s(k)T*g(k+1))    (conjugacy condition)
C                                                                    
C   where w and v are two positive parameters specified by the
C   user. Some suggested values are: w=7/8 and v=0.05.
C   The algorithm is not too much sensitive to these values.
C-------------------------------------------------------------------------
C                                                                  
C   The solution of the above system given by 1) and 2) is as follows:
C                             
C                            /                              \
C                  a(k)      |      (y(k)Ts(k))||g(k+1)||^2 |     b(k)
C   theta(k) = ------------- |1 + --------------------------| - ---------,
C               y(k)Tg(k+1)  |              delta(k)        |    delta(k)
C                            \                              /             
C
C                            /              \
C               y(k)Tg(k+1)  |       b(k)   |         ||g(k+1||^2
C   beta(k) =  ------------- |1 - ----------| + a(k) ------------- ,
C                y(k)Ts(k)   |     delta(k) |          delta(k)
C                            \              /
C
C   where
C
C   a(k) = v*(s(k)T*g(k+1)) + (y(k)T*g(k+1))
C
C   b(k) = w*||g(k+1)||^2*(s(k)T*y(k)) + (y(k)T*g(k+1))*(s(k)T*g(k+1))    
C
C   and
C
C   delta(k) = (s(k)T*g(k+1))*(y(k)T*g(k+1)) - (y(k)T*s(k))*||g(k+1)||^2.   
C
C   It is proved that the sequence delta(k) is uniformly bounded away from 
C   zero. On the other hand, if alpha is computed by the Wolfe line search.
C   then y(k)Ts(k)>0.
C             
C   w is a parameter for sufficient descent condition (w>0).
C   v is a parameter for conjugacy condition  (v>0).
C
C------------------------------------------------------------------------
C                                                          Neculai Andrei 
C                                      Research Institute for Informatics
C                           Center for Advanced Modeling and Optimization  
C                                                     and
C                                          Academy of Romanian Scientists
*************************************************************************
*
c********************************************************* END main program



c            *** Conjugate Gradient Algorithms Project *** 
C           ===============================================
c
c                                         
*
*
*-------------------------------------------------------------------
*                           Subroutine DESCON                                
*                         =====================
*                                   
*
*                             Neculai Andrei
*                  Research Institute for Informatics
*             Center for Advanced Modeling and Optimization
*              8-10, Averescu Avenue, Bucharest 1, Romania
*                        E-mail: nandrei@ici.ro
*                           voice: 666.58.70
*                                and
*                     Academy of Romanian Scientists 
*              Science and Information Technology Section              
*            54, Splaiul Independentei, Bucharest 5, Romania
*
*
*
* /-----------------------------------------------------------------\
* | DESCON is a subroutine dedicated to compute the minimizer of    |
* | a differentiable function with a large number of variables.     |
* |                                                                 |
* | This subroutine is accompanied by subroutine "LineSearch" which |
* | implements the Wolfe line search. Both these subroutines belong |
* | to DESCON package.                                              |
* |                                                                 |
* | The user must provide a subroutine to evaluate the function     |
* | value and its gradient in a point. The name of this subroutine  |
* | is EVALFG.                                                      |
* | The algebraic expression of the functions considered in this    |
* | program, as well as their Fortran code are presented into the   |
* | paper:                                                          |
* | N. Andrei, "An unconstrained optimization test functions        |
* | collection", Advanced Modeling and Optimization, vol.10, No.1,  |
* | (2008) pp.147-161.                                              | 
* | http://www.ici.ro/camo/journal/v10n1.htm                        |
* |                                                                 |
* | There are some facilities for the user to specify:              |
* | 1) The termination criterion.                                   |
* | 2) Convergence tolerance for gradient.                          |
* | 3) Convergence tolerance for function value.                    |
* | 4) Maximum number of iterations in LineSearch subroutine.       |
* | 5) The parameter w from the descent condition.                  |
* | 6) The parameter v from the conjugacy condition.                |
* |                                                                 |
* |-----------------------------------------------------------------|      
* |                                                                 |
* | The calling sequence of DESCON is:                              |
* |                                                                 |
* |    subroutine descon(n,x,epsg,maxiter,f,gnorm,stoptest,dc,cc,   |
* |                     iter,irs,fgcnt,lscnt,angle, powell,         |
* |                     ibeta, itheta, nexp)                        |
* |                                                                 |
* |Input parameters:                                                |
* |=================                                                |
* |n          (integer) number of variables.                        |
* |x          (double)  starting guess, length n. On output         |
* |                     contains the solution.                      |
* |epsg       (double)  convergence tolerance for gradient.         |
* |maxiter    (integer) maximum number of iterations.               |
* |stoptest = option parameter for selection of                     | 
* |            stopping criterion:                                  |
* |            if stoptest = 1 then consider the following test:    | 
* |               if(ginf .le. epsg)                                |
* |            if stoptest = 2 then consider the following test:    |
* |               if(gnorm .le. epsg)                               |
* |               where:                                            |
* |               ginf  = infinite norm of gradient g(xk),          | 
* |               gnorm = norm-2 of gradient g(xk).                 | 
* |dc        (double)  parameter for sufficient descent condition   |
* |cc        (double)  parameter for conjugacy condition            |
* |angle      (logical) parameter specifying the angle criterion of |
* |                     restart.                                    |
* |powell     (logical) parameter specifying the Powell criterion of|
* |                     restart.                                    |   
* |nexp       (integer) parameter specifying the number of the      |
* |                     problem considered in a train of experiments|
* |                                                                 |
* |                                                                 |
* |Output parameters:                                               |
* |==================                                               |
* |f          (double)  function value in final (optimal) point.    |
* |gnorm      (double)  norm-2 of gradient in final point.          |
* |iter       (integer) number of iterations to get the final point.|
* |irs        (integer) number of restart iterations.               |
* |fgcnt      (integer) number of function evaluations.             |
* |lscnt      (integer) number of line searches.                    |
* |ibeta      (integer) number of iterations in which beta is zero  |
* |itheta     (integer) number of iterations in which theta is one  |
* |-----------------------------------------------------------------|
* |                                                                 |
* |                                                                 |
* |Calling subroutines:                                             |
* |====================                                             | 
* |Subroutine DESCON is calling two subroutines:                    |
* |EVALFG     an user subroutine (function and gradient),           |
* |LINESEARCH a package subroutine.                                 |
* |                                                                 |   
* |The user must supply a subroutine with the function and its      |
* |gradient:                                                        |
* |  call funder(n,x,fx,grad,nexp)                                  |
* |where:                                                           |
* |  n    (integer)  number of variables.                           |
* |  x    (double)   the current iteration.                         |
* |  fx   (double)   function value in point x.                     |
* |  grad (double)   array with gradient of function in point x.    |
* |  nexp (integer)  parameter specifying the number of the         |
* |                  problem considered in a train of experiments.  |
* |                                                                 |
* |                                                                 |
* |The Wolfe line search is implemented in the subroutine           |
* |LINESEARCH, which belongs to the package.                        |
* |The calling sequence of this subroutine is as follows:           |            
* | call LineSearch (n,x,f,d,gtd,dnorm,alpha,xnew,fnew,gnew,        |
* |                  fgcnt,lscnt,lsflag, nexp)                      |
* |where:                                                           |
* |  n      (integer)  number of variables.                         |
* |  x      (double)   the current iteration.                       |
* |  f      (double)   function value in current point.             |
* |  d      (double)   array with search direction.                 |
* |  gtd    (double)   scalar: grad'*d.                             |   
* |  dnorm  (double)   2 norm of d.                                 |
* |  alpha  (double)   step length (given by the LineSearch).       |
* |  xnew   (double)   array with the new estimation of variables.  |
* |  fnew   (double)   function value in xnew.                      |
* |  gnew   (double)   array with gradient in xnew.                 |
* |  fgcnt  (integer)  number of function evaluations.              |
* |  lscnt  (integer)  number of line searches.                     |
* |  lsflag (integer)  parameter for abnormal Line Search           |
* |                    Termination. If the # of iterations in       |
* |                    LineSearch is greater than 20 then lsflag=1  |
* |  nexp   (integer)  parameter specifying the number of the       |
* |                    problem considered in a train of experiments |
* |                                                                 |
* |Subroutine LINESEARCH is the same as that used in CONMIN package |
* |by Shanno and Phua, and in SCALCG package by Andrei.             |
* |                                                                 |
* |-----------------------------------------------------------------|
* |                                           Neculai Andrei, 2010  |
* \-----------------------------------------------------------------/
*           
*
c
c********************************************************************

      subroutine descon(n,x,epsg,maxiter,f,gnorm,stoptest,dc,cc,
     *                  iter,irs,fgcnt,lscnt,angle, powell, 
     *                  ibeta, itheta, nexp)

      parameter(ia=1000000)

C     SCALAR ARGUMENTS
      integer n,iter,irs,fgcnt,lscnt,maxiter
      integer stoptest, nexp
      double precision epsg, f,gnorm     
      logical angle, powell 

C     ARRAY ARGUMENTS
      double precision x(n)

C     LOCAL SCALARS
      integer i,lsflag
      double precision fnew,alpha,gtg, 
     +        dnorm,dnormprev, ginf,
     +        gtd, gtgp, stg, acc,bdc,  
     +        yts, ytg, theta, delta, dc,cc, epsm, bn,sigma
      
C     LOCAL ARRAYS
      double precision xnew(ia),g(ia),gnew(ia),d(ia),
     +        y(ia), s(ia)
      common /acca/epsm
            
* Initialization
      
      n5 = mod(n,5)
      n6 = n5 + 1
      
      iter   = 0
      irs    = 0
      fgcnt  = 0
      lscnt  = 0         
      ibeta  = 0
      itheta = 0
      
      call funder(n,x,f,g, nexp)

      fgcnt = fgcnt + 1

        gtg = 0.0d0
        do i = 1,n5
          d(i) = - g(i)
          gtg  = gtg + g(i) ** 2
        end do    
        do i = n6,n,5
          d(i)   = -g(i)
          d(i+1) = -g(i+1)
          d(i+2) = -g(i+2)
          d(i+3) = -g(i+3)
          d(i+4) = -g(i+4)
          gtg  = gtg + g(i)**2 + g(i+1)**2 + g(i+2)**2 +
     *                           g(i+3)**2 + g(i+4)**2   
        end do
      gnorm = sqrt( gtg )


      gtd   = -gtg
      dnorm = gnorm                   

      if ( gnorm .ne. 0.0d0 ) then
          alpha = 1.0d0 / gnorm
      end if 
      
      write(*,10)
      write(*,20) epsg, n
      write(*,30) f, gnorm
      write(*,40)
      write(*,50)
 10   format('*************************************************')
 20   format(' gtol= ', 1pd10.3, ' n = ',i5)
 30   format(' Initial f= ',1pd10.3,'   gnorm= ',1pd10.3)
 40   format(' ')
 50   format(' iter fgcnt      f          gnorm        step    restart')

      sigma = 0.8d0       

C  ------------------------------   Main loop   --------------------    
C====================================================================

110    continue
      
c------------------------------------------------  STOP TEST

      if(iter .eq. 0) go to 91

        if(stoptest .eq. 1) then
          ginf=dabs(g(1))
          do i=2,n5
            if(dabs(g(i))   .gt. ginf) ginf = dabs(g(i))
          end do   
          do i=n6,n,5
            if(dabs(g(i))   .gt. ginf) ginf = dabs(g(i))
            if(dabs(g(i+1)) .gt. ginf) ginf = dabs(g(i+1))
            if(dabs(g(i+2)) .gt. ginf) ginf = dabs(g(i+2))
            if(dabs(g(i+3)) .gt. ginf) ginf = dabs(g(i+3))
            if(dabs(g(i+4)) .gt. ginf) ginf = dabs(g(i+4))
          end do  
          if(ginf .le. epsg) go to 999    
        end if
c
        if(stoptest .eq. 2) then
          if(gnorm .le. epsg) go to 999
        end if 
c      
91    continue
c------------------------------------------------ END STOP TEST
      

* Increment iteration

          iter = iter + 1  
          if(iter .gt. maxiter) go to 999

* Line search                                         
* Determine the step length ALPHA and the new point XNEW, as well as
* the function value in xnew, FNEW, and the gradient in xnew, GNEW.

          call LineSearch(n,x,f,d,gtd,dnorm,alpha,xnew,fnew,gnew,
     +                    sigma,fgcnt,lscnt,lsflag, n5,n6,nexp)

*
*---------------------------------- Acceleration section
C                                   ====================
c
c (We use an/bn only if b is different from zero)
c (an=gtd)

      bn=0.d0
      do i = 1,n5       
        bn = bn + (g(i)-gnew(i))*d(i)
      end do   
      do i = n6,n,5
        bn= bn + (g(i)-gnew(i))*d(i)+(g(i+1)-gnew(i+1))*d(i+1)+
     *           (g(i+2)-gnew(i+2))*d(i+2)+(g(i+3)-gnew(i+3))*d(i+3)+
     *           (g(i+4)-gnew(i+4))*d(i+4)   
      end do

      if(dabs(bn) .gt. epsm) then
        if(gtd/bn .ne. 1.0d0) then
          do i=1,n5
            xnew(i) = x(i) + (gtd/bn)*alpha*d(i)
          end do   
          do i=n6,n,5
            xnew(i)   = x(i)   + (gtd/bn)*alpha*d(i)
            xnew(i+1) = x(i+1) + (gtd/bn)*alpha*d(i+1)
            xnew(i+2) = x(i+2) + (gtd/bn)*alpha*d(i+2)
            xnew(i+3) = x(i+3) + (gtd/bn)*alpha*d(i+3)
            xnew(i+4) = x(i+4) + (gtd/bn)*alpha*d(i+4)
          end do
          call funder(n,xnew,fnew,gnew, nexp)
          fgcnt = fgcnt + 1
        end if  
      end if

*
*
*---------------------------------- Prepare some scalar products
*                                   
*              
          gtg = 0.d0
          gtgp= 0.d0          
          ytg = 0.d0 
          stg = 0.d0
          yts = 0.d0   
       do i = 1,n5    
         s(i) = xnew(i) - x(i)
         y(i) = gnew(i) - g(i)
         ytg = ytg + gnew(i)*y(i)
         stg = stg + gnew(i)*s(i)
         yts = yts + y(i)*s(i)
         gtgp = gtgp + gnew(i)*g(i)
           x(i) = xnew(i)
           g(i) = gnew(i)
         gtg  = gtg + g(i) * g(i)             
       end do  

       do i = n6,n,5
         s(i)   = xnew(i)   - x(i) 
         s(i+1) = xnew(i+1) - x(i+1)
         s(i+2) = xnew(i+2) - x(i+2)
         s(i+3) = xnew(i+3) - x(i+3)
         s(i+4) = xnew(i+4) - x(i+4)  
         y(i)   = gnew(i)   - g(i)
         y(i+1) = gnew(i+1) - g(i+1)
         y(i+2) = gnew(i+2) - g(i+2)
         y(i+3) = gnew(i+3) - g(i+3)
         y(i+4) = gnew(i+4) - g(i+4)
         ytg = ytg + gnew(i)*y(i)+gnew(i+1)*y(i+1)+gnew(i+2)*y(i+2)+
     *               gnew(i+3)*y(i+3)+gnew(i+4)*y(i+4)    
         stg = stg + gnew(i)*s(i)+gnew(i+1)*s(i+1)+gnew(i+2)*s(i+2)+
     *               gnew(i+3)*s(i+3)+gnew(i+4)*s(i+4)    
         yts = yts + y(i)*s(i)+y(i+1)*s(i+1)+y(i+2)*s(i+2)+
     *                         y(i+3)*s(i+3)+y(i+4)*s(i+4)
         gtgp = gtgp + gnew(i)*g(i)+gnew(i+1)*g(i+1)+gnew(i+2)*g(i+2)+
     *                gnew(i+3)*g(i+3)+gnew(i+4)*g(i+4)
           x(i)   = xnew(i)
           x(i+1) = xnew(i+1)
           x(i+2) = xnew(i+2)
           x(i+3) = xnew(i+3)
           x(i+4) = xnew(i+4)  
           g(i)   = gnew(i)
           g(i+1) = gnew(i+1)
           g(i+2) = gnew(i+2)
           g(i+3) = gnew(i+3)        
           g(i+4) = gnew(i+4)                                   
           gtg  = gtg + g(i)*g(i)+g(i+1)*g(i+1)+g(i+2)*g(i+2)+
     *                  g(i+3)*g(i+3)+g(i+4)*g(i+4)      
       end do
*    
          gnorm= sqrt( gtg )
*
          f = fnew
          dnormprev = dnorm  
*                         
* Sigma computation.
*
          sigma = gtg/(dabs(ytg)+gtg)
*
*
* -------------------------------- Delta, Theta and Beta computation  
*                                  =================================
*
*   Delta computation
*
          delta = stg*ytg-gtg*yts

*   acc (conjugacy condition)   (ak)
*   bdc (descent condition)     (bk)
                               
          acc = cc*stg + ytg
          bdc = dc*yts*gtg + ytg*stg
c
*   Beta and theta computation                  
*   If |delta| is greater than epsilon machine, then compute
*   beta and theta by means of the algorithm.        
*   Otherwise, set theta=1 and beta = 0.
*
*   Now theta computation:
         if(dabs(delta) .gt. epsm) then 
          theta = (acc/ytg)*(1.d0+yts*gtg/delta)-bdc/delta
         else
          theta = 1.d0 
          itheta=itheta+1
         end if  
c
c
* For beta computation we consider 3 possibilities:
* 1) beta as in algorithm DESCON,
* 2) beta as suggested by Dai and Liao+ (i.e. similar to PRP+),
* 3) beta truncated (see Hager and Zhang).             
*
* Therefore, the algorithm can be implemented in 4 variants as follows:
* a) (2.1), (2.2), (2.12) and (2.13) as it is considered in the Technical Report.
* b) (2.1), (2.2), (2.12) and (3.29).
* c) (2.1), (2.2), (2.12), (2.13) and truncation.
* d) (2.1), (2.2), (2.12), (3.29) and truncation.
* Please see ICI Technical Report, November 29, 2010.
* Numerical experiments proved that all these variants have similar performances.
*         
      if(yts .ne. 0.d0 .and. dabs(delta) .gt. epsm) then
        beta = ytg/yts-(bdc*ytg)/(yts*delta)+acc*gtg/delta
cc       beta = dmax1(ytg/yts,0.d0)-(bdc*ytg)/(yts*delta)+acc*gtg/delta
cc       beta = dmax1(beta,-1.d0/(dnorm*dmin1(0.1d0,gnorm)))
      else
        beta  = 0.d0              
        ibeta=ibeta+1
      end if  

*                        
* --------------------------------- Direction computation
*                                  =======================
*
         dnorm = 0.0d0
         gtd   = 0.0d0
         do i = 1,n5
           d(i)  = -theta*g(i) + beta*s(i)
           dnorm =   dnorm + d(i) ** 2
           gtd   =   gtd + g(i) * d(i)  
         end do
         
         do i = n6,n,5  
           d(i)    =-theta * g(i)   + beta*s(i) 
           d(i+1)  =-theta * g(i+1) + beta*s(i+1) 
           d(i+2)  =-theta * g(i+2) + beta*s(i+2) 
           d(i+3)  =-theta * g(i+3) + beta*s(i+3) 
           d(i+4)  =-theta * g(i+4) + beta*s(i+4) 

           dnorm   =   dnorm + d(i)**2+d(i+1)**2+d(i+2)**2+
     *                                 d(i+3)**2+d(i+4)**2
           gtd     =   gtd + g(i)*d(i)+g(i+1)*d(i+1)+g(i+2)*d(i+2)+
     *                                 g(i+3)*d(i+3)+g(i+4)*d(i+4) 
         end do

         dnorm = sqrt( dnorm )   
    
*       
*------------------------------------ END direction computation
*             
*   RESTART CRITERIA
*   ================
*
* Angle Restart Test
*
        if(angle) then
          if ( gtd .gt. -1.0d-03 * gnorm * dnorm ) then
              irs = irs + 1
              do i = 1,n
                d(i) = -g(i)
              end do
              dnorm =  gnorm
              gtd   = -gtg  
          end if        
        end if

*
* Beale-Powell restart test
*             
        if(powell) then
          if(dabs(gtgp) .gt. 0.2d0*dabs(gtg)) then
              irs = irs + 1
              do i = 1,n5
                d(i) = -g(i)
              end do     
              do i = n6,n,5
                d(i)   = -g(i)
                d(i+1) = -g(i+1)
                d(i+2) = -g(i+2)
                d(i+3) = -g(i+3)
                d(i+4) = -g(i+4)            
              end do              
              dnorm =   gnorm
              gtd   =  -gtg  
          end if            
        end if  
*------------------------------------------- Prepare first trial
c                                            of steplength
          
        if(dnorm .ne. 0.d0) then
          alpha = alpha * dnormprev / dnorm
        else
          alpha = 1.d0
        end if    
c
      write(*,80)iter,fgcnt,f,gnorm,(gtd/bn)*alpha,irs
 80   format(2(i4,1x),3x,3(1pd10.3,2x),i4)
      write(84,*)fgcnt,f,gnorm

      go to 110


*------------------------------------------ End of main loop

999   continue
      
*     
      return 

      end

*-------------------------------------------- END DESCON subroutine





c******************************************************************

      subroutine LineSearch (n,x,f,d,gtd,dnorm,alpha,xnew,fnew,gnew,
     +                       sigma,fgcnt,lscnt,lsflag, n5,n6,nexp)

C     This is the one-dimensional line search used in CONMIN

C     SCALAR ARGUMENTS
      integer n,fgcnt,lscnt,lsflag, nexp
      double precision f,gtd,dnorm,alpha,fnew

C     ARRAY ARGUMENTS
      double precision x(n),d(n),xnew(n),gnew(n)

C     LOCAL SCALARS
      integer i,lsiter
      double precision alphap,alphatemp,fp,dp,gtdnew,a,b,sigma

      lsflag = 0                                              
c      sigma =0.8d0
      
* Maximum number of LineSearch is max_ls (now=20)

      max_ls=10
      
      alphap = 0.0d0
      fp     = f
      dp     = gtd

      do i = 1,n5
        xnew(i) = x(i) + alpha * d(i)
      end do     
      do i = n6,n,5
        xnew(i)   = x(i)   + alpha * d(i)
        xnew(i+1) = x(i+1) + alpha * d(i+1)
        xnew(i+2) = x(i+2) + alpha * d(i+2)
        xnew(i+3) = x(i+3) + alpha * d(i+3)
        xnew(i+4) = x(i+4) + alpha * d(i+4)
      end do    
c1
         call funder(n,xnew,fnew,gnew, nexp)
         fgcnt = fgcnt + 1

      gtdnew = 0.0d0
      do i = 1,n5
       gtdnew = gtdnew + gnew(i) * d(i)
      end do     
      do i = n6,n,5
       gtdnew = gtdnew + gnew(i)*d(i)+gnew(i+1)*d(i+1)+gnew(i+2)*d(i+2)+
     *                   gnew(i+3)*d(i+3)+gnew(i+4)*d(i+4)
      end do
          
      lsiter = 0                                          

 10   if ( alpha * dnorm .gt. 1.0d-30 .and. lsiter .lt. max_ls .and.
     +   .not. ( gtdnew .eq. 0.0d0 .and. fnew .lt. f ) .and.
     +   ( ( fnew .gt. f + 1.0d-04 * alpha * gtd .or. 
     +   dabs( gtdnew / gtd ) .gt. sigma ) .or. ( lsiter .eq. 0 .and.
     +   dabs( gtdnew / gtd ) .gt. 0.5d0 ) ) ) then

 20       if ( alpha * dnorm .gt. 1.0d-30 .and. fnew .gt. f .and.
     +         gtdnew .lt. 0.0d0 ) then

              alpha = alpha / 3.0d0

              do i = 1,n5
                xnew(i) = x(i) + alpha * d(i)
              end do     
              do i = n6,n,5                
                xnew(i)   = x(i)   + alpha * d(i)
                xnew(i+1) = x(i+1) + alpha * d(i+1)
                xnew(i+2) = x(i+2) + alpha * d(i+2)
                xnew(i+3) = x(i+3) + alpha * d(i+3)
                xnew(i+4) = x(i+4) + alpha * d(i+4)
              end do  
c2
                call funder(n,xnew,fnew,gnew, nexp)
                fgcnt = fgcnt + 1

              gtdnew = 0.0d0
              do i = 1,n5
                gtdnew = gtdnew + gnew(i) * d(i)
              end do     
              do i = n6,n,5              
                gtdnew = gtdnew + gnew(i)*d(i)+gnew(i+1)*d(i+1)+
     *                            gnew(i+2)*d(i+2)+gnew(i+3)*d(i+3)+
     *                            gnew(i+4)*d(i+4)     
              end do

              alphap = 0.0d0
              fp     = f
              dp     = gtd

              goto 20

          end if
                 
          a = dp + gtdnew - 3.0d0 * ( fp - fnew ) / ( alphap - alpha )
          b = a ** 2 - dp * gtdnew

          if ( b .gt. 0.0d0 ) then
              b = sqrt( b )
          else
              b = 0.0d0
          end if

          alphatemp = alpha - ( alpha - alphap ) * ( gtdnew + b - a ) /
     +                ( gtdnew - dp + 2.0d0 * b )

          if ( gtdnew / dp .le. 0.0d0 ) then

              if ( 0.99d0 * max( alpha, alphap ) .lt. alphatemp .or.
     +            alphatemp .lt. 1.01d0 * min( alpha, alphap ) ) then
                  alphatemp = ( alpha + alphap ) / 2.0d0
              end if

          else

              if ( gtdnew .lt. 0.0d0 .and. 
     +            alphatemp .lt. 1.01d0 * max( alpha, alphap ) ) then
                  alphatemp = 2.0d0 * max( alpha, alphap )
              end if

              if ( ( gtdnew .gt. 0.0d0 .and.
     +            alphatemp .gt. 0.99d0 * min( alpha, alphap ) ) .or.
     +            alphatemp .lt. 0.0d0 ) then
                  alphatemp = min( alpha, alphap ) / 2.0d0
              end if

          end if

          alphap = alpha
          fp     = fnew
          dp     = gtdnew

          alpha = alphatemp

          do i = 1,n5
            xnew(i) = x(i) + alpha * d(i)
          end do                           
          do i = n6,n,5
            xnew(i)   = x(i)   + alpha * d(i)
            xnew(i+1) = x(i+1) + alpha * d(i+1)
            xnew(i+2) = x(i+2) + alpha * d(i+2)
            xnew(i+3) = x(i+3) + alpha * d(i+3)
            xnew(i+4) = x(i+4) + alpha * d(i+4)
          end do   
c3
            call funder(n,xnew,fnew,gnew, nexp)
            fgcnt = fgcnt + 1

          gtdnew = 0.0d0
          do i = 1,n5
            gtdnew = gtdnew + gnew(i) * d(i)
          end do                              
          do i = n6,n,5
            gtdnew = gtdnew + gnew(i)*d(i)+gnew(i+1)*d(i+1)+
     *                        gnew(i+2)*d(i+2)+gnew(i+3)*d(i+3)+     
     *                        gnew(i+4)*d(i+4)
          end do

          lsiter = lsiter + 1

      goto 10

      end if

      if ( lsiter .ge. max_ls ) then
          lsflag = 1
      end if

      if ( lsiter .ne. 0 ) then
          lscnt = lscnt + 1
      end if

      return

      end
*---------------------------------- End LineSearch subroutine

                                           
                                           


                                
                                
*-----------------------------------------------------------
*  Date created       : May 30, 1995
*  Date last modified : May 30, 1995
*
*  Subroutine for execution time computation.
*
*-----------------------------------------------------------
*
      subroutine exetim(tih,tim,tis,tic, tfh,tfm,tfs,tfc)
*
      integer*4 tih,tim,tis,tic
      integer*4 tfh,tfm,tfs,tfc
*
      integer*4 ti,tf
      integer*4 ch,cm,cs
      data ch,cm,cs/360000,6000,100/
*
      ti=tih*ch+tim*cm+tis*cs+tic
      tf=tfh*ch+tfm*cm+tfs*cs+tfc
      tf=tf-ti
      tfh=tf/ch
      tf=tf-tfh*ch
      tfm=tf/cm
      tf=tf-tfm*cm
      tfs=tf/cs
      tfc=tf-tfs*cs
*
      return
      end
*---------------------------------------------- End of EXETIM
