module shared

    implicit none

    !   fixed parameters
    integer, parameter :: maxiter = 300

    !   size variables
    integer :: n, m, k_start, k_end, k_max
    namelist /sizedata/ n, m, k_start, k_end, k_max

    !   helper variables
    integer :: ns, nctr, nt, nobs

    !   state variables
    double precision, dimension(:,:,:), allocatable :: u, v, phi

    !   observation variables
    double precision, dimension(:,:,:), allocatable :: uobs, vobs, phiobs

    !   exact values (to be used for validation)
    double precision, dimension(:,:,:), allocatable :: uexact, vexact, phiexact

    !   adjoint variables
    double precision, dimension(:,:,:), allocatable :: uadj, vadj, phiadj

    !   observation errors
    double precision :: UObsStddev, VObsStddev, PhiObsStddev
    namelist /obs_errors/ UObsStddev, VObsStddev, PhiObsStddev

    !   background values
    double precision, dimension(:,:), allocatable :: uback, vback, phiback

    !   square root background error covariance
    double precision, dimension(:,:), allocatable :: sqrtPfU, sqrtPfV, sqrtPfPhi

    !   test case variables
    integer :: ObsOpNum, OptimizerNum, ObsOpSmoothing
    double precision :: delta, CovLocalizationLength, &
                        ObsOpSmoothingKU, ObsOpSmoothingKV, ObsOpSmoothingKPhi

    namelist /test_cases/ ObsOpNum, OptimizerNum, ObsOpSmoothing, &
                        ObsOpSmoothingKU, ObsOpSmoothingKV, ObsOpSmoothingKPhi, &
                        CovLocalizationLength, delta

    !   perturbation variables
    integer :: NoMembers

    double precision :: DX, DY, CorrH, CovInflation, UBackgroundStddev, VBackgroundStddev, &
                      PhiBackgroundStddev
    namelist /perturbation/ NoMembers, DX, DY, CorrH, CovInflation, &
                          UBackgroundStddev, VBackgroundStddev, &
                          PhiBackgroundStddev

    !   RMSE variables
    double precision :: urmse, vrmse, phirmse, rmseroot

contains

    subroutine initialize()

        implicit none

    !   local variables
        integer :: i, j, k, ierr

    !   read from namelists
        open(unit=42,file="namelist.swm",status='old')
        read(42,sizedata)
        close(42)

        open(unit=42,file="namelist.swm",status='old')
        read(42,perturbation)
        close(42)

        open(unit=42,file="namelist.swm",status='old')
        read(42,obs_errors)
        close(42)

        open(unit=42,file="namelist.swm",status='old')
        read(42,test_cases)
        close(42)

    !   initialize helper variables

        ns = (n-2)*(m-2)
        nobs = ns
        nctr = 3*ns
        nt = k_max

        rmseroot = sqrt(1.*(n-2)*(m-2))

    !   allocate state variables
        allocate (u(n,m,nt), STAT=ierr)
        if(ierr /= 0) write(*,*) "allocation error for u"

        allocate (v(n,m,nt), STAT=ierr)
        if(ierr /= 0) write(*,*) "allocation error for v"

        allocate (phi(n,m,nt), STAT=ierr)
        if(ierr /= 0) write(*,*) "allocation error for phi"

    !   allocate observation variables
        allocate (uobs(n,m,nt), STAT=ierr)
        if(ierr /= 0) write(*,*) "allocation error for uobs"

        allocate (vobs(n,m,nt), STAT=ierr)
        if(ierr /= 0) write(*,*) "allocation error for vobs"

        allocate (phiobs(n,m,nt), STAT=ierr)
        if(ierr /= 0) write(*,*) "allocation error for phiobs"

    !   allocate exact variables
        allocate (uexact(n,m,nt), STAT=ierr)
        if(ierr /= 0) write(*,*) "allocation error for uexact"

        allocate (vexact(n,m,nt), STAT=ierr)
        if(ierr /= 0) write(*,*) "allocation error for vexact"

        allocate (phiexact(n,m,nt), STAT=ierr)
        if(ierr /= 0) write(*,*) "allocation error for phiexact"

    !   allocate adjoint variables
        allocate (uadj(n,m,nt), STAT=ierr)
        if(ierr /= 0) write(*,*) "allocation error for uadj"

        allocate (vadj(n,m,nt), STAT=ierr)
        if(ierr /= 0) write(*,*) "allocation error for vadj"

        allocate (phiadj(n,m,nt), STAT=ierr)
        if(ierr /= 0) write(*,*) "allocation error for phiadj"

    !   allocate background values
        allocate (uback(n-2,m-2), STAT=ierr)
        if(ierr /= 0) write(*,*) "allocation error for uback"

        allocate (vback(n-2,m-2), STAT=ierr)
        if(ierr /= 0) write(*,*) "allocation error for vback"

        allocate (phiback(n-2,m-2), STAT=ierr)
        if(ierr /= 0) write(*,*) "allocation error for phiback"

    !   allocate sqrt background covariance values
        allocate (sqrtPfU(ns,ns), STAT=ierr)
        if(ierr /= 0) write(*,*) "allocation error for sqrtPfU"

        allocate (sqrtPfV(ns,ns), STAT=ierr)
        if(ierr /= 0) write(*,*) "allocation error for sqrtPfV"

        allocate (sqrtPfPhi(ns,ns), STAT=ierr)
        if(ierr /= 0) write(*,*) "allocation error for sqrtPfPhi"

        uobs = 0
        vobs = 0
        phiobs = 0
        u = 0
        v = 0
        phi = 0
        uadj = 0
        vadj = 0
        phiadj = 0
        uexact = 0
        vexact = 0
        phiexact = 0
        uback = 0
        vback = 0
        phiback = 0

    end subroutine initialize

    subroutine readFiles()
        implicit none

    !   local variables
        double precision :: ensfactor
        integer :: i, j, k, ierr

        print *,'Now reading the u, v and phi sqrt background error covariance matrix (B^(1/2)) values'

        open(unit=12,file='HorzCorrU.txt',status='unknown', &
          form='formatted')
        read(12,*) ((sqrtPfU(i,j),i=1,ns),j=1,ns)
        close(12)

        open(unit=12,file='HorzCorrV.txt',status='unknown', &
          form='formatted')
        read(12,*) ((sqrtPfV(i,j),i=1,ns),j=1,ns)
        close(12)

        open(unit=12,file='HorzCorrP.txt',status='unknown', &
          form='formatted')
        read(12,*) ((sqrtPfPhi(i,j),i=1,ns),j=1,ns)
        close(12)

    !   scale sqrtPfU, sqrtPfV and sqrtPfPhi by the number of ensemble members used
    !   in estimating it.  The covariance decreases by this factor due to the Central Limit Theorem
        ensfactor = sqrt(NoMembers + 0.)

       sqrtPfU = sqrtPfU/ensfactor
       sqrtPfV = sqrtPfV/ensfactor
       sqrtPfPhi = sqrtPfPhi/ensfactor

        print *,'Now reading the u, v and phi background values'

        open(unit=12,file='SWM_mean.txt',status='unknown', &
           form='formatted')
        read(12,*) ((uback(i,j),i=1,n-2),j=1,m-2), &
                   ((vback(i,j),i=1,n-2),j=1,m-2), &
                   ((phiback(i,j),i=1,n-2),j=1,m-2)
        close(12)

    !   read observations from file
        open(unit=10,file='SWM_obs.dat',status='unknown', &
           form='formatted')
        read(10,*) (((uobs(i,j,k),i=1,n),j=1,m),k=1,k_max), &
                 (((vobs(i,j,k),i=1,n),j=1,m),k=1,k_max), &
                 (((phiobs(i,j,k),i=1,n),j=1,m),k=1,k_max)
        close(10)

        open(unit=10,file='SWM_exact.txt',status='unknown', &
           form='formatted')
        read(10,*) (((uexact(i,j,k),i=1,n),j=1,m),k=1,k_max), &
                 (((vexact(i,j,k),i=1,n),j=1,m),k=1,k_max), &
                 (((phiexact(i,j,k),i=1,n),j=1,m),k=1,k_max)
        close(10)
    end subroutine

    subroutine finalize()
        implicit none

    !   deallocate state variables
        deallocate(u)
        deallocate(v)
        deallocate(phi)

    !   deallocate observation variables
        deallocate(uobs)
        deallocate(vobs)
        deallocate(phiobs)

    !   deallocate exact variables
        deallocate(uexact)
        deallocate(vexact)
        deallocate(phiexact)

    !   deallocate adjoint variables
        deallocate(uadj)
        deallocate(vadj)
        deallocate(phiadj)

    !   deallocate background variables
        deallocate(uback)
        deallocate(vback)
        deallocate(phiback)

    !   deallocate sqrt background covariance values
        deallocate(sqrtPfU)
        deallocate(sqrtPfV)
        deallocate(sqrtPfPhi)
    
    end subroutine finalize

    function map_3to1(i,j,k,NiX,NiY)

        implicit none

        integer::i,j,k,NiX,NiY
        integer::map_3to1
        map_3to1=i+NiX*(j-1)+NiX*NiY*(k-1)

    end function map_3to1
  
    function map_2to1(i,j,Nix)

        implicit none

        integer::i,j,NiX
        integer::map_2to1
        
        map_2to1=i+NiX*(j-1)
 
    end function map_2to1

    subroutine map_1to2(x,NiX,i,j)

        implicit none

        integer,intent(in)::x,NiX
        integer,intent(out)::i,j
        
        j=INT((x-1)/NiX)+1
        i=x-(j-1)*NiX
    
    end subroutine map_1to2

    subroutine mean(RR,max,mn)

        implicit none

        double precision::RR(*)
        double precision::mn
        integer::max,i
        double precision::sum
        sum=0.0d0
        
        do i=1,max
            sum=sum+RR(i)
        end do
        mn=sum/max

    end subroutine mean
  
    subroutine std(RR,max,mn,sd)
        implicit none

        double precision::RR(*)
        double precision::mn,sd
        integer::max,i
        double precision::sum
        
        sd=0.0
        do i=1,max
            sd=sd+(mn-RR(i))**2
        end do
        sd=sd/(max-1)
        sd=sqrt(sd)

    end subroutine std
  
    function random_normal() result (ran_norm)
    
    ! Adapted from the following Fortran 77 code
    !      ALGORITHM 712, COLLECTED ALGORITHMS FROM ACM.
    !      THIS WORK PUBLISHED IN TRANSACTIONS ON MATHEMATICAL SOFTWARE,
    !      VOL. 18, NO. 4, DECEMBER, 1992, PP. 434-435.
    
    !  The function random_normal() returns a normally distributed pseudo-random
    !  number with zero mean and unit variance.   This version uses the default
    !  uniform random number generator which is in your fortran library.
    
    !  The algorithm uses the ratio of uniforms method of A.J. Kinderman
    !  and J.F. Monahan augmented with quadratic bounding curves.
    
    !  Fortran 90 version by Alan Miller (alan @ mel.dms.csiro.au)
    
        implicit none
        double precision :: ran_norm
        
        !     Local variables
        double precision, parameter :: s = 0.449871, t = -0.386595, a = 0.19600, b = 0.25472,   &
             half = 0.5, r1 = 0.27597, r2 = 0.27846
        double precision::ru,rv
        double precision             :: u, v, x, y, q
        
        !     Generate P = (u,v) uniform in rectangle enclosing acceptance region
        do
            call random_number(ru)
            call random_number(rv)
            u=dble(ru)
            v=dble(rv)
            v = 1.7156 * (v - half)
           
            !     Evaluate the quadratic form
            x = u - s
            y = abs(v) - t
            q = x**2 + y*(a*y - b*x)
           
            !     Accept P if inside inner ellipse
            if (q < r1) exit
            !     Reject P if outside outer ellipse
            if (q > r2) cycle
            !     Reject P if outside acceptance region
            if (v**2 < -4.0*log(u)*u**2) exit
        END DO
        
        !     Return ratio of P's coordinates as the normal deviate
        ran_norm = v/u
        return
    end function random_normal

    subroutine calculateRmse(k)
        implicit none

        integer, intent(in) :: k
        integer :: i,j

        urmse = 0.
        vrmse = 0.
        phirmse = 0.

        do j=2,m-1
            do i=2,n-1
                urmse = urmse + (u(i,j,k) - uexact(i,j,k))**2
                vrmse = vrmse + (v(i,j,k) - vexact(i,j,k))**2
                phirmse = phirmse + (phi(i,j,k) - phiexact(i,j,k))**2
            enddo
        enddo

        print *, 'RMSE(',k,'):', sqrt(urmse)/rmseroot, &
                               sqrt(vrmse)/rmseroot, &
                               sqrt(phirmse)/rmseroot

    end subroutine

    subroutine alloc1(toAlloc,N,name)
        implicit none

        double precision, dimension(:), intent(out), allocatable :: toAlloc
        integer, intent(in) :: N
        character(len = *), intent(in) :: name

        integer :: ierr

        allocate(toAlloc(N),stat=ierr)

        if (ierr.ne.0) then
            write(*,*) "Allocation error:", name
            STOP
        end if

    end subroutine

    subroutine alloc2(toAlloc,N,M,name)
        implicit none

        double precision, dimension(:,:), intent(out), allocatable :: toAlloc
        integer, intent(in) :: N, M
        character(len = *), intent(in) :: name

        integer :: ierr

        allocate(toAlloc(N,M),stat=ierr)

        if (ierr.ne.0) then
            write(*,*) "Allocation error:", name
            STOP
        end if

    end subroutine

    subroutine alloc3(toAlloc,N,M,O,name)
        implicit none

        double precision, dimension(:,:,:), intent(out), allocatable :: toAlloc
        integer, intent(in) :: N, M, O
        character(len = *), intent(in) :: name

        integer :: ierr

        allocate(toAlloc(N,M,O),stat=ierr)

        if (ierr.ne.0) then
            write(*,*) "Allocation error:", name
            STOP
        end if

    end subroutine

    subroutine alloc4(toAlloc,N,M,O,P,name)
        implicit none

        double precision, dimension(:,:,:,:), intent(out), allocatable :: toAlloc
        integer, intent(in) :: N, M, O, P
        character(len = *), intent(in) :: name

        integer :: ierr

        allocate(toAlloc(N,M,O,P),stat=ierr)

        if (ierr.ne.0) then
            write(*,*) "Allocation error:", name
            STOP
        end if
    end subroutine

endmodule shared
