      program swe_getInit
!     *************************************************************************
!     Program to get the initial conditions for the shallow water equations.  
!     The IC values are from the paper "A Survey of Finite-Difference Schemes 
!     for the Primitive Equations for a Barotropic Fluid", Grammeltvedt, A.  
!     Monthly Weather Review, Vol. 97, Issue 5, Page 384, 1969.  
!     DOI: 10.1175/1520-0493(1969)097<0384:ASOFSF>2.3.CO;2
!
!     A perturbation is then added to the initial conditions, and this is used 
!     as the starting point for minimization.
!     *************************************************************************
      use shared
      implicit none 

! local variables
      integer :: i, j, ierr, k
      double precision :: uv, vv, phiv

      call initialize()

! initialize u,v,phi ic
      call initana(u, v, phi, n, m, nt)

! evolve model in time from exact IC
      call swe_model(u(:,:,1:nt), v(:,:,1:nt), phi(:,:,1:nt), n, m, nt, 1, 1, nt)

! write out observationns 
      open(unit=9,file='SWM_exact.txt',status='unknown', &
           form='formatted')
      write(9,*) (((u(i,j,k),i=1,n),j=1,m),k=1,nt), &
                 (((v(i,j,k),i=1,n),j=1,m),k=1,nt), &
                 (((phi(i,j,k),i=1,n),j=1,m),k=1,nt)
      close(9)

      open(unit=9,file='SWM_ini.txt',status='unknown', &
           form='formatted')
      write(9,*) (((u(i,j,k),i=1,n),j=1,m),k=1,1), &
                 (((v(i,j,k),i=1,n),j=1,m),k=1,1), &
                 (((phi(i,j,k),i=1,n),j=1,m),k=1,1)
      close(9)

      do i=1,n
          do j=1,m
              do k=1,nt
                  call obs_operator(u(i,j,k),v(i,j,k),phi(i,j,k),uv,vv,phiv)

                  uobs(i,j,k) = uv
                  vobs(i,j,k) = vv
                  phiobs(i,j,k) = phiv
              end do
          end do
      end do

      do i=1,n
          do j=1,m
              do k=1,nt
                  uobs(i,j,k) = uobs(i,j,k) + UObsStddev*random_normal()
                  vobs(i,j,k) = vobs(i,j,k) + VObsStddev*random_normal()
                  phiobs(i,j,k) = phiobs(i,j,k) + PhiObsStddev*random_normal()
              end do
          end do
      end do


      open(unit=9,file='SWM_obs.dat',status='unknown', &
           form='formatted')
      write(9,*)  (((uobs(i,j,k),i=1,n),j=1,m),k=1,nt), &
                  (((vobs(i,j,k),i=1,n),j=1,m),k=1,nt), &
                  (((phiobs(i,j,k),i=1,n),j=1,m),k=1,nt)
      close(9)

! deallocate variables
      call finalize()

! end of program
      end

      subroutine initana(u, v, phi, n, m, nt)

!     *************************************************************************
!     Setup the initial conditions in u(:,:,1), v(:,:,1), phi(:,:,1). The
!     initial conditions are from Grammeltvedt.  The wind components are
!     derived from the geopotential.
!     *************************************************************************

      implicit none


! size parameters
      integer, intent(in) :: n, m, nt

! state variables.  Only the first time step will be written.
      double precision, intent (inout), dimension(n, m, nt) :: u, v, phi

! length, depth and heights in m
      double precision :: length, depth, h0, h1, h2

! gravity in m/s^2
      double precision :: gp
      data gp/10./

! Coriolis in 1/s
      double precision :: f
      data f/1d-4/

! Beta-plane, 1/(m s)
      double precision :: beta
      data beta/1.5d-11/

! Namelist for initial condition parameter data
      namelist /icdata/ length, depth, h0, h1, h2

      double precision :: pi
      data pi/3.14159/

! Local variables
      double precision :: dx, dy, hx, hy, hhx, hhy
      double precision :: xix, triarg, xtrig, dtrig
      double precision :: yiy, fr, co
      double precision :: ab1, bb, dco, derth
      double precision :: aa, dersec
      integer :: i, j

! read IC parameters from namelist
      open(unit=42,file="namelist.swm",status='old',form='formatted')
      read(42,icdata)
      close(42)

! Finite difference variables
      dx = (2.*length)/(n - 1)
      dy = (2.*depth)/(m - 1)
      hx = 1./(4.*dx)
      hy = 1./(4.*dy)
      hhx = 2.*hx
      hhy = 2.*hy

! Set initial conditions
      do 10 i = 1, n
          do 20 j = 1, m
              xix = (i - 1)*dx/2.
              triarg = 2.*pi*xix/length
              xtrig = sin(triarg)
              dtrig = (2.*pi/length)*cos(triarg)

              yiy = (j - 1)*dy/2.
              fr = f + beta*(yiy-depth/2.)

              co = 9.*((depth/2.) - yiy)/(2.*depth)

              ab1 = cosh(co)
              bb = 1./(ab1*ab1)
              dco = -4.5/depth
              derth = bb*dco

              aa = tanh(co)
              dersec = -2.*aa*derth

              u(i, j, 1) = -gp*(h1*derth + h2*xtrig*dersec )/fr
              v(i, j, 1) = gp*(h2*bb*dtrig)/fr
              phi(i, j, 1) = gp*(h0 + h1*aa + h2*bb*xtrig)
20        continue
10    continue

      return
      end

